SystemOrganization addCategory: #'MonticelloFileTree-Git'!

MCVersionDependency subclass: #MCResolvedVersionDependency
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCResolvedVersionDependency commentStamp: '<historical>' prior: 0!
This version dependency entry contains the specific version we depend on.

On normal use, upon encountering a dependency, Monticello will re-ask all repositories to load the dependency, instead of just the current repository.

In GitFileTree, due to the --depth parameter, version numbering may differ, so we have to work around Monticello for this scheme to work.

If Monticello was properly selecting packages per UUID, then it would probably not be necessary.!

!MCResolvedVersionDependency class methodsFor: 'instance-creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
package: aPackage fromGit: aCommitID in: aRepository
	^ self basicNew initializeWithPackage: aPackage fromGit: aCommitID in: aRepository! !

!MCResolvedVersionDependency methodsFor: 'initialization' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
initializeWithPackage: aPackage fromGit: aCommitID in: aRepository
	package := aPackage.
	version := aRepository versionFromPackageName: package name commit: aCommitID.
	versionInfo := version info! !

!MCResolvedVersionDependency methodsFor: 'resolving' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
resolve
	"Ok, consider that we will be resolved only once. We could use a become: to stop being a resolved version instead."

	| v |
	version ifNil: [ ^ super version ].
	v := version.
	version := nil.
	^ v! !

!MCResolvedVersionDependency methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
version
	^ version! !

!MCResolvedVersionDependency methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
version: anObject
	version := anObject! !

MCFileTreeRepository subclass: #MCFileTreeVcsRepository
	instanceVariableNames: 'remoteUrl branch subdirectory ancestry'
	classVariableNames: 'NamesMap VcsCommand'
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeVcsRepository commentStamp: '<historical>' prior: 0!
A MCFileTreeVcsRepository adds VCS commands via OSProcess when saving a package in a filetree.!

MCFileTreeVcsRepository subclass: #MCFileTreeGitRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitRepository commentStamp: '<historical>' prior: 0!
A MCFileTreeGitRepository adds git commands via OSProcess when saving a package in a filetree.
Uses gitfiletree: as a protocol (:().

A gitfiletree url with a protocol parameter is the target for a remote. Otherwise the url is considered as a local file reference.
	Parameters are:
		dir : the directory inside the repository where the target MC packages are.
		branch : the git branch to fetch.
		protocol : the protocol to use to access the repository from git. Essentially make the difference between ssh (git@hostname:pathToRepo) and others urls (https://hostname/pathToRepo).
		readOnly : is the repository read only? If yes, reduce the history to a minimum, restrict operations on the GUI and append the created repository name.
	Once parameters are set, it becomes mandatory to have a protocol and a remote url
	Among parameters, protocol is mandatory, all others are optional.
	
Example :
	MCFileTreeGitRepository fromZnUrl: (ZnUrl fromString: 'gitfiletree://github.com/ThierryGoubier/filetree.git?protocol=git&dir=repository&branch=pharo3.0' ).
	Creates a MC repository on a git clone of github.com/ThierryGoubier/filetree.git,
	on branch pharo3.0
	stored in a directory named filetree under the Pharo working directory,
	and pointing to the repository/ subdirectory where the filetree packages  are kept.

gitfiletree core documentation :

gitfiletree maps MC commands onto git commands via OSProcess, and MC metadata out of git commit data. That's all there is to it. A bit of git knowledge can help, but no git command line work is necessary. The inner workings are a bit more complex than that of course, but not by much.!

MCFileTreeGitRepository subclass: #MCFileTreeGitRemoteRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitRemoteRepository commentStamp: '<historical>' prior: 0!
A MCFileTreeGitRemoteRepository is just there to get in the creation part of the stuff. The main code is in the superclass.

Will fix that later, don't want to bother about it now.!

!MCFileTreeGitRemoteRepository class methodsFor: 'instance creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
creationTemplate
	^ 'MCFileTreeGitRemoteRepository
	location: ''git@''
	name: ''''
	subdirectory: ''''
	branch: '''''! !

!MCFileTreeGitRemoteRepository class methodsFor: 'instance creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
description
	^ 'Remote git repository'! !

!MCFileTreeGitRemoteRepository class methodsFor: 'instance creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
fillInTheBlankRequest
	^ 'GIT Repository:'
			! !

!MCFileTreeGitRemoteRepository class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 16:18'!
location: aGitUrl name: aName subdirectory: aDirectoryName branch: aBranch
	"Create this directly."

	| repo |
	repo := self new.
	repo
		subdirectory:
			(aDirectoryName isEmpty
				ifTrue: [ nil ]
				ifFalse: [ aDirectoryName ]).
	repo
		branch:
			(aBranch isEmpty
				ifTrue: [ nil ]
				ifFalse: [ aBranch ]).
	repo remoteUrl: aGitUrl.
	repo readonly: false.
	repo vcsCloneRepositoryAndCheckoutIn: aName.
	^ repo! !

!MCFileTreeGitRemoteRepository class methodsFor: 'instance creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
morphicConfigure
	^ self fillInTheBlankConfigure! !

!MCFileTreeGitRepository class methodsFor: 'instance creation' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
description
	^ 'gitfiletree://'! !

!MCFileTreeGitRepository class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 16:18'!
fromZnUrl: aZnUrl
	"Let hack that. Fed up with the lack of progress at times, can't work."

	"A gitfiletree url with a host is the target for a remote. All other parameters are optional.
	Parameters are:
		dir : the directory inside the repository where the target MC packages are.
		branch : the git branch to fetch.
		protocol: the user name part to add to the ssh Url, default to git, but can also be https (which implies read only access).
		readOnly : is the repository read only? If present, reduce the history to a minimum (and change the GUI)."

	| repo |
	repo := self new.
	self assert: aZnUrl scheme = #gitfiletree.
	aZnUrl hasHost
		ifTrue: [ 
			| protocol |
			repo subdirectory: (aZnUrl queryAt: 'dir' ifAbsent: [  ]).
			repo branch: (aZnUrl queryAt: 'branch' ifAbsent: [  ]).
			protocol := aZnUrl queryAt: 'protocol' ifAbsent: [ 'git' ].
			repo readonly: (aZnUrl queryAt: 'readOnly' ifAbsent: [  ]) notNil.
			protocol = 'git'
				ifTrue: [ repo remoteUrl: 'git@' , aZnUrl host , ':' , aZnUrl path ].
			protocol = 'https'
				ifTrue: [ 
					repo remoteUrl: 'https://' , aZnUrl host , '/' , aZnUrl path.
					repo readonly: true ].
			repo
				vcsCloneRepositoryAndCheckoutIn:
					(aZnUrl lastPathSegment ifNil: [ aZnUrl host ] ifNotNil: [ :n | n asFileReference basenameWithoutExtension ]) ]
		ifFalse: [ repo directory: aZnUrl asGitFileReference ].
	^ repo! !

!MCFileTreeGitRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:10'!
runOSProcessVcsCommand: anArrayOfStrings in: aDirectory
	| c r d in out err |
	[ 
	in := ExternalPipe nonBlockingPipe.
	out := ExternalPipe nonBlockingPipe.
	err := ExternalPipe nonBlockingPipe.
	c := PipeableOSProcess externalProxyClass
		programName: self vcsCommand
		arguments: anArrayOfStrings
		initialEnvironment: nil.
	c initialStdIn: in reader.
	c initialStdOut: out writer.
	c initialStdErr: err writer.
	c pwd: (MCFileTreeFileUtils current directoryPathString: aDirectory).
	c := c initialize value.
	d := nil.
	r := out upToEnd.
	[ 
	c update: #runState.
	c isComplete ]
		whileFalse: [ 
			(d ifNil: [ d := Delay forMilliseconds: 5 ]) wait.
			r := r , out upToEnd ].
	r := r , out upToEndOfFile.
	c succeeded
		ifFalse: [ 
			| errorString |
			errorString := err upToEndOfFile.
			errorString notEmpty
				ifTrue: [ self error: 'Git error: ' , errorString ].
			r := '' ] ]
		ensure: [ 
			in close.
			out close.
			err close ].
	^ r! !

!MCFileTreeGitRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:10'!
runProcessWrapperVcsCommand: anArrayOfStrings in: aDirectory
	"Enclose all parameters with double quotes to protect."

	| r aCommandString |
	aCommandString := String
		streamContents: [ :stream | 
			anArrayOfStrings
				do: [ :e | 
					stream
						nextPut: $";
						nextPutAll: e;
						nextPutAll: '" ' ] ].
	(Smalltalk at: #ProcessWrapper ifAbsent: [ self error: 'Please load ProcessWrapper' ])
		ifNotNil: [ :pW | 
			| command |
			command := pW new
				useStdout;
				useStderr;
				startWithCommand:
					self vcsCommand , ' -C "'
						,
							(MCFileTreeFileUtils current directoryPathString: aDirectory)
						, '" ' , aCommandString;
				yourself.
			command waitForExit.
			self assert: command isRunning not.
			r := command upToEnd.
			command exitCode > 0
				ifTrue:
					[ 
					| errorString |
					errorString := command errorUpToEnd.
					errorString notEmpty
						ifTrue:
							[ MCFileTreeGitError new signal: 'Git error: ' , errorString ].
					r := '' ] ].
	^ r
! !

!MCFileTreeGitRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:10'!
runVcsCommand: anArrayOfStrings in: aDirectory
	"Dispatch depending on the platform."

	Smalltalk platform isWin32
		ifTrue: [ ^ self runProcessWrapperVcsCommand: anArrayOfStrings in: aDirectory ]
		ifFalse: [ ^ self runOSProcessVcsCommand: anArrayOfStrings in: aDirectory ]! !

!MCFileTreeGitRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:09'!
vcsCommand
	"Preferences for Squeak as well."
	
	"Platform determination is probably Pharo-specific."

	<preference: 'Git Command Path'
		category: 'monticello'
		description: 'The complete path for the git command, if not in the default PATH and different in name'
		type: #String>

	^ VcsCommand ifNil: [ Smalltalk os isWindows ifTrue: [ 'git' ] ifFalse:[ '/usr/bin/git' ] ]
	
! !

!MCFileTreeGitRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:09'!
vcsCommand: aPath
	VcsCommand := aPath! !

!MCFileTreeGitRepository class methodsFor: 'settings' stamp: 'HilaireFernandes 10/24/2015 16:10'!
vcsPerPackageProperties
	^ '.gitfiletree'! !

!MCFileTreeGitRepository methodsFor: 'storing' stamp: 'HilaireFernandes 10/24/2015 16:16'!
basicStoreVersion: aVersion
	"Do a filetree store followed by in order, a check and save dependencies, a commit and an update of the version info."

	"Recover the last commit id and update the version info to have a correctly synchronized in image id (apart from the version number?)"

	| packageDirectoryString output deps gitCommand |
	self internalStoreVersion: aVersion.
	deps := self internalStoreDependencies: aVersion.
	packageDirectoryString := Array
		streamContents: [ :stream | 
			{aVersion} , deps
				do: [ :each | 
					stream
						nextPut: (self class parseName: each info name) first, self packageExtension ] ].
	gitCommand := Array
		streamContents: [ :stream | 
			stream nextPutAll: #('commit' '--allow-empty-message').
			(self class namesAt: aVersion info author)
				ifNotNil: [ :n | stream nextPut: '--author=' , (n at: #name) , ' ', (n at: #email) ].
			stream
				nextPut: '-m';
				nextPut: (self escapeForShell: aVersion info message convertToSystemString);
				nextPut: '--';
				nextPutAll: packageDirectoryString ].
	self vcsCommand: gitCommand in: directory.
	output := (self vcsCommand: #('log' '-1' '--format=%H') in: directory) substrings last.
	{aVersion} , deps
		do: [ :each | 
			each info
				id:
					(GitFileTreePackageEntry
						uuidFromVcsSHA: (ByteArray readHexFrom: output)
						package: (aVersion info name copyFrom: 1 to: (aVersion info name lastIndexOf: $-) - 1)) ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:16'!
currentBranch
	"How do we handle a detached head status?"

	| branches indexOfStar current |
	branches := self vcsBranchs.
	indexOfStar := branches indexOf: '*'.
	current := branches at: indexOfStar + 1.
	(current first = $( and: [ current last ~= $) ])
		ifTrue: [ 
			(branches allButFirst: indexOfStar + 1)
				do: [ :each | 
					current := current , ' ' , each.
					each last = $)
						ifTrue: [ ^ current ] ].
			^ current ]
		ifFalse: [ ^ current ]! !

!MCFileTreeGitRepository methodsFor: 'descriptions' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
description
	directory exists
		ifFalse: [ ^ 'deleted' ].
	remoteUrl ifNil: [ ^ self shortDirectoryDescription , ' [' , self branch , '] (GitFileTree)' ].
	self branch ifNotNil: [ ^ self shortRemoteDescription , ' [' , self branch , '] (Remote Git)' ].
	^ remoteUrl! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
goferVersionFrom: aVersionReference
	"gitfiletree:// provides all version names as found by git."

	"Should be part of gofer, but isn't."

	"Since gofer is text based, we can't give him git entries, but strings and those are then rematched to get the right entry."

	| mcR packageName entry |
	packageName := (self class parseName: aVersionReference name) first , self packageExtension.
	mcR := MCFileTreeGitStReader on: directory fileName: packageName.
	mcR commitID: (entry := self allFileNames detect: [ :e | e name = aVersionReference name ]) commitID.
	mcR info: entry info.
	mcR repository: self.
	^ mcR version! !

!MCFileTreeGitRepository methodsFor: 'storing' stamp: 'HilaireFernandes 10/24/2015 16:16'!
internalStoreVersion: aVersion
	"Do a filetree store followed by a git add command."

	"Do not commit yet. Commit will only happens after checking the dependencies."

	| packageDirectoryString |
	super basicStoreVersion: aVersion.
	packageDirectoryString := (self class parseName: aVersion info name) first , self packageExtension.
	aVersion dependencies notEmpty
		ifTrue: [ self writeGitFileTreeProperties: (self fileUtils directoryFromPath: packageDirectoryString relativeTo: directory) ].
	self
		vcsCommand:
			{'add'.
			packageDirectoryString}
		in: directory! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:18'!
loadAllFileNames
	"When reading the file names, generate GitFileTreePackageEntry objects with the path and the info."

	"Algorithm: may move in a dedicated scanner/parser in the future:
	input :
		line *;
	line :
		log_size log_data ;
	log_size :
		'log size' <integer> ;
	log_data :
		<SHA1> '|' user_name '|' time_stamp '|' message ;
	End of message can be obtained by the log_size entry.
	"

	| allNames entries commitIDs anAncestry |
	allNames := OrderedCollection new.
	entries := self directory entries select: [ :entry | entry isDirectory and: [ self canReadFileNamed: entry name ] ].
	'loading packages versions from git'
		displayProgressFrom: 0
		to: entries size
		during: [ :bar | 
			anAncestry := self loadAncestry.
			1 to: entries size do: [ :i | 
				(entries at: i)
					in: [ :entry | 
						| entryVersions v |
						entryVersions := OrderedCollection new.
						commitIDs := self vcsVersionsForPackage: entry name.
						[ commitIDs atEnd ]
							whileFalse: [ 
								| logSize commitID author position message dateAndTime |
								(commitIDs match: 'log size ')
									ifFalse: [ self error: 'Unable to find git data in input' ].
								logSize := Integer readFrom: commitIDs.
								commitIDs next.
								position := commitIDs position.
								commitID := ByteArray readHexFrom: (commitIDs upTo: $|) trimBoth.
								author := (commitIDs upTo: $|) onlyLetters.
								dateAndTime := DateAndTime fromString: (commitIDs upTo: $|).
								message := (commitIDs next: logSize + 1 + position - commitIDs position) convertFromSystemString.
								entryVersions
									add:
										(GitFileTreePackageEntry
											on: entry name
											id: commitID
											author: author
											message: message
											timeStamp: dateAndTime) ].
						v := 1.
						entryVersions size to: 1 by: -1 do: [ :j | 
							(entryVersions at: j)
								buildInfoWith: entryVersions
								startingAt: j + 1
								version: v
								ancestry: anAncestry.
							v := v + 1 ].
						allNames addAll: entryVersions ].
				bar value: i ] ].
	^ allNames! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:17'!
loadAncestry
	"Load the ancestry chain to be able to scan it properly."

	| s |
	s := self vcsVersions.
	ancestry := Dictionary new.
	(s match: 'commit ')
		ifFalse: [ self error: 'Unable to find git data in input' ].
	[ s atEnd ]
		whileFalse: [ 
			| a |
			a := (s upToAll: 'commit') substrings collect: [ :e | ByteArray readHexFrom: e trimBoth ].
			ancestry at: a first put: a allButFirst ].
	^ ancestry! !

!MCFileTreeGitRepository methodsFor: 'interface' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
morphicOpen: aWorkingCopy
	(MCFileTreeGitRepositoryInspector repository: self workingCopy: aWorkingCopy) show! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
packageDescriptionFromPackageEntry: packageEntry
	| filename extension packageDirectory |
	packageDirectory := self fileDirectoryOn: packageEntry.
	filename := self fileUtils current directoryName: packageDirectory.
	extension := filename copyFrom: (filename lastIndexOf: $.) to: filename size.
	^ ((self packageExtension ~= '.package'
		and: [ 
			(self fileUtils filePathExists: 'version' relativeTo: packageDirectory)
				and: [ self fileUtils filePathExists: 'package' relativeTo: packageDirectory ] ])
		or: [ 
			| dir |
			dir := self fileUtils directoryFromPath: MCFileTreeStCypressWriter monticelloMetaDirName relativeTo: packageDirectory.
			self fileUtils directoryExists: dir ])
		ifTrue: [ self class parseEntry: packageEntry extension: extension ]
		ifFalse: [ 
			{(filename copyFrom: 1 to: (filename lastIndexOf: $.) - 1).
			'cypress'.
			1.
			packageEntry} ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
packageDescriptionsFromReadableFileNames
	^ (self readableFileNames
		select: [ :packageEntry | self fileUtils directoryExists: (self fileDirectoryOn: packageEntry) ])
			collect: [ :packageEntry | self packageDescriptionFromPackageEntry: packageEntry ]! !

!MCFileTreeGitRepository methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
repositoryProperties
	repositoryProperties
		ifNil: [ 
			repositoryProperties := Dictionary new.
			(self fileUtils directoryExists: directory)
				ifFalse: [ 
					self
						error:
							'gitfiletree:// repository ' , (self fileUtils directoryPathString: self directory) printString , ' does not exist.' ].
			(self directory entries detect: [ :entry | entry name = '.filetree' ] ifNone: [  ])
				ifNil: [ 
					repositoryProperties := self defaultRepositoryProperties.
					self writeRepositoryProperties ]
				ifNotNil: [ :configEntry | configEntry readStreamDo: [ :fileStream | repositoryProperties := MCFileTreeJsonParser parseStream: fileStream ] ] ].
	^ repositoryProperties! !

!MCFileTreeGitRepository methodsFor: 'descriptions' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
shortDirectoryDescription
	^ ((directory fileSystem resolve: directory path) segments last: 2) in: [ :s | s first , '/' , s last ]! !

!MCFileTreeGitRepository methodsFor: 'descriptions' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
shortRemoteDescription
	(remoteUrl beginsWith: 'git@')
		ifTrue: [ ^ remoteUrl allButFirst: (remoteUrl indexOf: $:) ]
		ifFalse: [ ^ remoteUrl asZnUrl segments in: [ :s | (s last: 2) first , '/' , s last ] ]! !

!MCFileTreeGitRepository methodsFor: 'storing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
storeDependencies: aVersion
	"It's a no-op: it has been done in basicStoreVersion"! !

!MCFileTreeGitRepository methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
tag
	^ '^6c776b42^'! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:16'!
vcsBranchs
	"Query git for all the branches in the repository"

	"This is used to determine the current branch."

	| s |
	s := (self vcsCommand: #('branch') in: directory) substrings.
	^ s! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:18'!
vcsCloneRepositoryAndCheckoutIn: aDirectoryName
	"Do a clone on the remote repository and do a checkout on it to get the right branch. Complement the directory as well with the working directory. Note the name change when the directory is read-only: this is to protect of a Metacello problem when loading from a configuration with both a read-only and a read-write target (do I also need to protect when dealing with branches?)."

	"Metacello issues: even when selecting a specific version, will I load all repositories? It seems no."

	"Interesting. I can't use relative paths with MCFileTreeUtils"

	| targetDir targetDirName |
	targetDirName := readonly
		ifTrue: [ aDirectoryName , '-readonly' ]
		ifFalse: [ aDirectoryName ].
	targetDir := targetDirName asFileReference.
	targetDir exists
		ifTrue: [ 
			| targetDirBranch |
			self
				directory:
					(subdirectory isNil
						ifTrue: [ targetDir ]
						ifFalse: [ targetDir / subdirectory ]).
			targetDirBranch := self currentBranch.
			((branch isNil and: [ targetDirBranch = 'master' ]) or: [ targetDirBranch = branch ])
				ifFalse: [ 
					self
						error:
							'GitFileTree target directory already exists and is on another branch, cancelling clone and repository instanciation : '
								, (self fileUtils directoryPathString: targetDir) ] ]
		ifFalse: [ 
			self
				vcsCommand:
					#('clone')
						,
							(branch
								ifNil: [ #() ]
								ifNotNil: [ 
									{'-b'.
									branch} ])
						,
							(readonly
								ifTrue: [ #('--depth' '1') ]
								ifFalse: [ #() ])
						,
							{remoteUrl.
							targetDirName}
				in: FileSystem workingDirectory.
			GrowlMorph openWithLabel: 'gitfiletree://' contents: 'Cloning remote repository: done!!'.
			self
				directory:
					(subdirectory isNil
						ifTrue: [ targetDir ]
						ifFalse: [ targetDir / subdirectory ]) ]! !

!MCFileTreeGitRepository methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:16'!
vcsCommand: aCommandString in: aDirectory
	^ self class runVcsCommand: aCommandString in: aDirectory! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsNeedPush
	"Check if we need a push. Is that message multilingual ?"

	^ (self vcsCommand: #('log' '--branches' '--not' '--remotes' '--oneline') in: directory) size > 1! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPull
	"Do a pull --rebase on the repository."

	"The pull may fail because this amounts to a merge and rebase (after having done a fetch) and hence may end up in conflicts, that the user will have to solve by hand. Perspectives are that with a merge-tool written in Pharo, we will be able to fall back to Pharo to resolve the conflict, which should be very cool."

	self vcsCommand: #('pull' '--rebase') in: directory! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPush
	"Do a push on the repository"

	self vcsCommand: #('push') in: directory! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsVersions
	"Query git for all the commits in the repository"

	"This is used to establish an ancestry chain package per package."

	^ ReadStream on: (self vcsCommand: #('log' '--format=commit %H %P') in: directory)! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsVersionsForDefinition: aMCDefinition in: aPackageName
	"Query git for all the versions related to that method definition."

	| fileDirectory methodFile |
	fileDirectory := self fileUtils
		directoryFromPath:
			aMCDefinition className
				,
					(aMCDefinition isExtensionMethod
						ifTrue: [ '.extension' ]
						ifFalse: [ '.class' ])
		relativeTo: (self fileUtils directoryFromPath: aPackageName , '.package' relativeTo: directory).
	methodFile := (aMCDefinition classIsMeta
		ifTrue: [ 'class' ]
		ifFalse: [ 'instance' ]) , '/' , (MCFileTreeStCypressWriter new fileNameForSelector: aMCDefinition selector) , '.st'.
	(self fileUtils directoryExists: fileDirectory)
		ifFalse: [ NotFound signalFor: fileDirectory ].
	^ (self vcsCommand: #('log' '--format=%H') , {methodFile} in: fileDirectory) substrings
		collect: [ :e | ByteArray readHexFrom: e ]! !

!MCFileTreeGitRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:18'!
vcsVersionsForPackage: aDirectoryEntry
	"Query git for all the commits related to that directory entry. Typical use: the package top-level directory entry, i.e. <pathOfRepository>/<packageName>.package. Return a stream of git sha1, the commit author, the date and time of commit, and the message for each commit changing something in this directory entry."

	| packageDirectory |
	packageDirectory := self fileUtils directoryFromPath: aDirectoryEntry relativeTo: directory.
	^ ReadStream on: (self vcsCommand: #('log' '--log-size' '--format=%H|%an|%ai|%s' '--' '.') in: packageDirectory)! !

!MCFileTreeGitRepository methodsFor: 'as yet unclassified' stamp: 'HilaireFernandes 10/24/2015 16:20'!
versionFromFileNamed: aVcsFileTreePackageEntry
	self assert: aVcsFileTreePackageEntry class == GitFileTreePackageEntry.
	^ super versionFromFileNamed: aVcsFileTreePackageEntry! !

!MCFileTreeGitRepository methodsFor: 'i/o' stamp: 'HilaireFernandes 10/24/2015 16:16'!
writeRepositoryProperties
	"When writing the repository properties, commit them as well."

	super writeRepositoryProperties.
	self vcsCommand: #('add' '.filetree') in: directory.
	self vcsCommand: #('commit' '-m' 'gitfiletree repository properties') in: directory! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:05'!
defaultDirectoryName
	^ FileSystem workingDirectory fullName! !

!MCFileTreeVcsRepository class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 16:02'!
fromZnUrl: aZnUrl
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
monticelloMetaDirName
    ^ MCFileTreeStCypressWriter monticelloMetaDirName! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
namesAt: aName
	^ self namesAt: aName ifAbsent: [  ]! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
namesAt: aName ifAbsent: aBlock
	"To avoid creating the names map if empty"

	^ NamesMap ifNil: aBlock ifNotNil: [ NamesMap at: aName ifAbsent: aBlock ]! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
namesAt: aName put: aNameString email: anEmailString
	self assert: (aNameString matchesRegex: '(:isAlphaNumeric:|[ _.])+').
	self assert: (anEmailString matchesRegex: '<(:isAlphaNumeric:|\.)+\@(:isAlphaNumeric:|\.)+>').
	self namesMap
		at: aName
		put:
			{(#name -> aNameString).
			(#email -> anEmailString)} asDictionary! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
namesMap
	^ NamesMap ifNil: [ NamesMap := Dictionary new ]! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:07'!
namesMap: aDictionary
	NamesMap := aDictionary! !

!MCFileTreeVcsRepository class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 16:07'!
parseEntry: aPackageEntry extension: extension
	"picked up from GoferVersionReference>>parseName:"

	| basicName package branch author versionNumber packageName string |
	string := aPackageEntry printString.
	basicName := string last isDigit
		ifTrue: [ string ]
		ifFalse: [ (string copyUpToLast: $.) copyUpTo: $( ].
	package := basicName copyUpToLast: $-.
	(package includes: $.)
		ifFalse: [ branch := '' ]
		ifTrue: [ 
			branch := '.' , (package copyAfter: $.).
			package := package copyUpTo: $. ].
	author := (basicName copyAfterLast: $-) copyUpToLast: $..
	versionNumber := (basicName copyAfterLast: $-) copyAfterLast: $..
	(versionNumber notEmpty and: [ versionNumber allSatisfy: [ :each | each isDigit ] ])
		ifTrue: [ versionNumber := versionNumber asNumber ]
		ifFalse: [ versionNumber := 0 ].
	packageName := package , branch.
	^ {packageName.
	author.
	versionNumber.
	aPackageEntry}! !

!MCFileTreeVcsRepository class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 16:07'!
parseName: aString extension: extension
	"picked up from GoferVersionReference>>parseName:"

	| basicName package branch author versionNumber packageName |
	basicName := aString last isDigit
		ifTrue: [ aString ]
		ifFalse: [ (aString copyUpToLast: $.) copyUpTo: $( ].
	package := basicName copyUpToLast: $-.
	(package includes: $.)
		ifFalse: [ branch := '' ]
		ifTrue: [ 
			branch := '.' , (package copyAfter: $.).
			package := package copyUpTo: $. ].
	author := (basicName copyAfterLast: $-) copyUpToLast: $..
	versionNumber := (basicName copyAfterLast: $-) copyAfterLast: $..
	(versionNumber notEmpty and: [ versionNumber allSatisfy: [ :each | each isDigit ] ])
		ifTrue: [ versionNumber := versionNumber asNumber ]
		ifFalse: [ versionNumber := 0 ].
	packageName := package , branch.
	^ {packageName.
	author.
	versionNumber.
	basicName}! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:10'!
runOSProcessVcsCommand: anArrayOfStrings in: aDirectory
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:10'!
runProcessWrapperVcsCommand: anArrayOfStrings in: aDirectory
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:09'!
runVcsCommand: anArrayOfStrings in: aDirectory
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:09'!
vcsCommand
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:09'!
vcsCommand: aPath
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'settings' stamp: 'HilaireFernandes 10/24/2015 16:10'!
vcsPerPackageProperties
	self subclassResponsibility! !

!MCFileTreeVcsRepository class methodsFor: 'settings' stamp: 'HilaireFernandes 10/24/2015 16:09'!
vcsSettingsOn: aBuilder
	<systemsettings>
	(aBuilder group: #monticello)
		with: [ 
			(aBuilder setting: #vcsCommand)
				type: #File;
				label: 'Git command path' translated;
				target: self;
				description: 'The path to the git command if not in the default external command path' translated ]! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:11'!
allFileNames
	"Switch to caching for speed."

	cacheFileNames == true
		ifFalse: [ ^ self loadAllFileNames ].
	^ allFileNames ifNil: [ allFileNames := self loadAllFileNames ]! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:11'!
branch
	^ branch ifNil: [ branch := self currentBranch ]! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:11'!
branch: anObject
	branch := anObject! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:11'!
cacheAllFileNames
	"Switch to caching for speed."

	cacheFileNames = true! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:02'!
currentBranch
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:12'!
escapeForShell: aString
	^ aString class
		new: aString size
		streamContents: [ :stream | 
			| readStream char |
			readStream := aString readStream.
			[ readStream atEnd ]
				whileFalse: [ 
					char := readStream next.
					char = $"
						ifTrue: [ stream nextPut: $\ ].
					stream nextPut: char ] ]! !

!MCFileTreeVcsRepository methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:13'!
fileDirectoryOn: aVcsFileTreePackageEntry
	"We are dealing with VcsFileTreePackageEntry objects, not FileReferences."

	^ self fileUtils directoryFromPath: aVcsFileTreePackageEntry packageDirectory relativeTo: self directory! !

!MCFileTreeVcsRepository methodsFor: 'storing' stamp: 'HilaireFernandes 10/24/2015 16:13'!
internalStoreDependencies: aVersion
	| deps |
	deps := OrderedCollection new.
	MCCacheRepository uniqueInstance
		cacheAllFileNamesDuring: [ 
			self
				cacheAllFileNamesDuring: [ 
					aVersion
						allAvailableDependenciesDo: [ :dep | 
							(self includesVersionNamed: dep info name)
								ifFalse: [ 
									self internalStoreVersion: dep.
									deps add: dep ] ] ] ].
	^ deps! !

!MCFileTreeVcsRepository methodsFor: 'storing' stamp: 'HilaireFernandes 10/24/2015 16:02'!
internalStoreVersion: aVersion
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:02'!
loadAncestry
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:02'!
packageDescriptionFromPackageEntry: packageEntry
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:14'!
readableFileNames
	"Updated for gitfiletree use of packageEntry, not file names."

	^ self allFileNames! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:13'!
remoteUrl
	^ remoteUrl! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:13'!
remoteUrl: anObject
	remoteUrl := anObject! !

!MCFileTreeVcsRepository methodsFor: 'descriptions' stamp: 'HilaireFernandes 10/24/2015 16:02'!
shortDirectoryDescription
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'descriptions' stamp: 'HilaireFernandes 10/24/2015 16:02'!
shortRemoteDescription
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:14'!
subdirectory
	^ subdirectory! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:14'!
subdirectory: anObject
	subdirectory := anObject! !

!MCFileTreeVcsRepository methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:02'!
tag
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:16'!
vcsBranchs
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:18'!
vcsCloneRepositoryAndCheckoutIn: aDirectoryName
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:14'!
vcsCommand
	^ self class vcsCommand! !

!MCFileTreeVcsRepository methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:16'!
vcsCommand: aCommandString in: aDirectory
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsNeedPush
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPull
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPush
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsVersions
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsVersionsForDefinition: aMCDefinition in: aPackageName
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'git querying' stamp: 'HilaireFernandes 10/24/2015 16:18'!
vcsVersionsForPackage: aDirectoryEntry
	self subclassResponsibility! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:20'!
versionFromFileNamed: aVcsFileTreePackageEntry
	"In the GUI, we should have version names (contributeur, version number, branch) for aString instead of the filetree package directory itself. Handle both cases for the moment."

	"Will change when we will be reading the version directly from the repo. This change is underway and should work."

	| mcR packageName |
	self assert: aVcsFileTreePackageEntry class == GitFileTreePackageEntry.
	packageName := (self class parseName: aVcsFileTreePackageEntry info name) first , self packageExtension.
	mcR := MCFileTreeGitStReader on: directory fileName: packageName.
	mcR commitID: aVcsFileTreePackageEntry commitID.
	mcR info: aVcsFileTreePackageEntry info.
	mcR repository: self.
	^ mcR version! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:18'!
versionFromPackageName: aPackageName commit: aCommitID
	"Specific: we count on the fact that the ancestry instance variable contains the git ancestry of the last loadAllFileNames call."
	
	"Explanation for code using the ancestry: if, when storing a dependency along with the package (single commit), that dependency is exactly the same as a previous store, git considers that it hasn't changed and gitfiletree correctly answers that there is no version of the dependency for that commit. We then have to search through the ancestry to find an older commit for that dependency."

	| entry mcR packageName entries |
	packageName := aPackageName , self packageExtension.
	entries := self allFileNames select: [ :each | each packageDirectory = packageName ].
	entry := entries detect: [ :each | each commitID = aCommitID ] ifNone: [  ].
	entry
		ifNil: [ 
			| cid |
			cid := (ancestry at: aCommitID) first.
			[ (entry := entries detect: [ :each | each commitID = cid ] ifNone: [  ]) isNil ]
				whileTrue: [ cid := (ancestry at: cid) first ] ].
	mcR := MCFileTreeGitStReader on: directory fileName: packageName.
	mcR commitID: entry commitID.
	mcR info: entry info.
	mcR repository: self.
	^ mcR version! !

!MCFileTreeVcsRepository methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:18'!
versionNameFromFileName: ea
	^ ea name! !

!MCFileTreeVcsRepository methodsFor: 'i/o' stamp: 'HilaireFernandes 10/24/2015 16:15'!
writeGitFileTreeProperties: aPackageDirectory
	"Write git file tree properties with a package."

	self fileUtils
		writeStreamFor: self class vcsPerPackageProperties
		in: (self fileUtils directoryFromPath: 'monticello.meta' relativeTo: aPackageDirectory)
		do: [ :fileStream | 
			fileStream lineEndConvention: #lf.
			fileStream nextPutAll: '{'.
			fileStream
				nextPut: $";
				nextPutAll: 'SingleCommitForDependencies';
				nextPutAll: '" : "';
				nextPutAll: 'true';
				nextPut: $".
			fileStream nextPutAll: ' }' ]! !

!MetacelloPlatform methodsFor: '*monticellofiletree-git-override' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
createRepository: aRepositorySpec
	| type |
	type := aRepositorySpec type.
	aRepositorySpec description asZnUrl scheme = #gitfiletree
		ifTrue: [ ^ MCFileTreeGitRepository fromZnUrl: aRepositorySpec description asZnUrl ].
	type = 'http'
		ifTrue: [ 
			^ MCHttpRepository
				location: aRepositorySpec description
				user: aRepositorySpec username
				password: aRepositorySpec password ].
	type = 'directory'
		ifTrue: [ ^ MCDirectoryRepository new directory: (self fileHandleOn: aRepositorySpec description) ].
	Smalltalk
		at: #MCFileTreeRepository
		ifPresent: [ :cl | 
			type = 'filetree'
				ifTrue: [ 
					| description headerSize |
					description := aRepositorySpec description.
					headerSize := 'filetree://' size.
					^ cl new
						directory: (self fileHandleOn: (aRepositorySpec description copyFrom: headerSize + 1 to: description size)) ] ].
	Smalltalk
		at: #MCGitHubRepository
		ifPresent: [ :cl | 
			type = 'github'
				ifTrue: [ ^ cl location: aRepositorySpec description ] ].
	type = 'dictionary'
		ifTrue: [ 
			| description headerSize globalName |
			description := aRepositorySpec description.
			headerSize := 'dictionary://' size.
			globalName := (description copyFrom: headerSize + 1 to: description size) asSymbol.
			^ Smalltalk
				at: globalName
				ifAbsent: [ 
					Smalltalk
						at: globalName
						put:
							(MCDictionaryRepository new
								description: description;
								yourself) ] ].
	^ nil! !

!ZnUrl class methodsFor: '*monticellofiletree-git-override' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
defaultPortForScheme: scheme
	(#(#file #gitfiletree) includes: scheme)
		ifTrue: [ ^ nil ].
	(#(#http #ws) includes: scheme)
		ifTrue: [ ^ 80 ].
	(#(#https #wss) includes: scheme)
		ifTrue: [ ^ 443 ].
	(ZnUnknownScheme scheme: scheme) signal! !

!ZnUrl methodsFor: '*monticellofiletree-git-override' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
asGitFileReference
	"Convert the receiver into a new FileReference object.
	Note that for a FileReference a trailing slash is not relevant"

	| path |
	self isSlash
		ifTrue: [ ^ FileSystem root ].
	path := self isDirectoryPath
		ifTrue: [ segments allButLast ]
		ifFalse: [ segments copy ].
	^ FileReference fileSystem: FileSystem disk path: (AbsolutePath withAll: path)! !

!ZnUrl methodsFor: '*monticellofiletree-git-override' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
mcRepositoryAsUser: usernameString withPassword: passwordString
	self isHttp | self isHttps
		ifTrue: [ ^ MCHttpRepository location: self asString user: usernameString password: passwordString ].
	self isFile
		ifTrue: [ ^ MCDirectoryRepository new directory: self asFileReference ].
	self scheme = #filetree
		ifTrue: [ ^ MCFileTreeRepository new directory: self asFileReferenceWithRelativePath ].
	self scheme = #ftp
		ifTrue: [ 
			^ MCFtpRepository
				host:
					(self hasPort
						ifTrue: [ self host , ':' , self port asString ]
						ifFalse: [ self host ])
				directory: self path
				user: usernameString
				password: passwordString	"MCFtpRepository assumes NO prefixed / in the path" ].
	self scheme = #gitfiletree
		ifTrue: [ ^ MCFileTreeGitRepository fromZnUrl: self ].
	self error: ('No Monticello repository for URL scheme {1}' format: {(self scheme)})! !

MCFileRepositoryInspector subclass: #MCFileTreeGitRepositoryInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitRepositoryInspector commentStamp: '<historical>' prior: 0!
This code is there to adapt the GUI to the FileTreeGitRepository.!

!MCFileTreeGitRepositoryInspector methodsFor: 'morphic ui' stamp: 'HilaireFernandes 10/24/2015 16:17'!
buttonSpecs
	repository readonly
		ifTrue: [ ^ super buttonSpecs ]
		ifFalse: [ ^ #(#('Pull' #vcsPull 'git pull and refresh') #('Push' #vcsPush 'git push' #vcsNeedPush)) , super buttonSpecs ]! !

!MCFileTreeGitRepositoryInspector methodsFor: 'actions' stamp: 'HilaireFernandes 10/24/2015 16:17'!
refresh
	super refresh.
	self changed: #vcsNeedPush! !

!MCFileTreeGitRepositoryInspector methodsFor: 'morphic ui' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
saveChanges
	"Ensure we do refresh. A git repository is a local beast, so it is ok to call a refresh."

	super saveChanges.
	self refresh! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
setRepository: aFileBasedRepository workingCopy: aWorkingCopy
	order := self class order.
	repository := aFileBasedRepository.
	aWorkingCopy
		ifNil: [ 
			selectedPackage := self packageList isEmpty
				ifFalse: [ self packageList first asString ] ]
		ifNotNil: [ 
			selectedPackage := aWorkingCopy ancestry ancestorString copyUpToLast: $-.
			selectedPackage ifEmpty: [ selectedPackage := aWorkingCopy package name ] ].
	[ 
	self refresh.
	MCWorkingCopy addDependent: self ] fork! !

!MCFileTreeGitRepositoryInspector methodsFor: 'actions' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsNeedPush
	"Do a git push. A refresh isn't needed. It may fail..."
	
	^ repository vcsNeedPush! !

!MCFileTreeGitRepositoryInspector methodsFor: 'actions' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPull
	"Do a git pull. A refresh isn't needed. It may fail..."
	
	repository vcsPull.
	self refresh! !

!MCFileTreeGitRepositoryInspector methodsFor: 'actions' stamp: 'HilaireFernandes 10/24/2015 16:17'!
vcsPush
	"Do a git push. A refresh isn't needed. It may fail..."

	repository vcsPush.
	self changed: #vcsNeedPush! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
version
	^ version
		ifNil: [ 
			Cursor wait
				showWhile: [ version := repository versionFromFileNamed: (versions detect: [ :e | e last name = selectedVersion ]) last ].
			version ]! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
versionHighlight: aPackageEntry
	"Changed to take in account the fact the version name is in the package entry."

	"Should correct a bug in FileTree display: current package is allways in bold."

	| verName |
	self assert: aPackageEntry class == GitFileTreePackageEntry.
	verName := aPackageEntry info name.
	^ Text
		string: verName
		attribute:
			(TextEmphasis new
				emphasisCode:
					((loaded includes: verName)
						ifTrue: [ 4	"underlined" ]
						ifFalse: [ 
							(aPackageEntry info breadthFirstAncestors anySatisfy: [ :v | loaded includes: v name ])
								ifFalse: [ 0 ]
								ifTrue: [ 1	"bold" ] ]))! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
versionInfo
	^ (Cursor wait showWhile: [ (versions detect: [ :e | e last name = selectedVersion ]) last ]) info! !

!MCFileTreeGitRepositoryInspector methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
versionSummary
	^ (Cursor wait showWhile: [ (versions detect: [ :e | e last name = selectedVersion ]) last ]) summary! !

MCFileTreeStCypressReader subclass: #MCFileTreeGitStReader
	instanceVariableNames: 'commitID repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!MCFileTreeGitStReader commentStamp: '<historical>' prior: 0!
This class handles reading a package from a git (via a specific commit ID).

Uses OS-Process to call git archive and retrieve a zip of the package. Then sort of reuse the superclass implementation (MczReader?) even if the StCypress format is different.

Remember:
	* stream is the directory holding the .package directory.
	* Package directory is NameOfPackage.package.
	* zip contains the archive of the package (the contents of NameOfPackage.package according to the commitID)
	
Complex work-arounds to handle incorrect incoming streams; see parseMember:

Avoid reading the version metadata : consider the MCVersionInfo is given by the GUI before loading the package. Do not read any metadata outside of the zip archive, since we know that the repository inspector won't use us for reading the metadata.

shouldNotImplement methods are here to protect against program paths I haven't been able to cover in the superclass, or changes in the superclass implementation. It may indicate that the superclass isn't the right one.!

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addClassAndMethodDefinitions
	(self zip members select: [ :m | m isDirectory and: [ m splitFileName last endsWith: '.trait' ] ])
		do: [ :entry | self addTraitAndMethodDefinitionsFromEntry: entry ].
	(self zip members select: [ :m | m isDirectory and: [ m splitFileName last endsWith: '.class' ] ])
		do: [ :entry | self addClassAndMethodDefinitionsFromEntry: entry ].
	(self zip members select: [ :m | m isDirectory and: [ m splitFileName last endsWith: '.extension' ] ])
		do: [ :entry | self addExtensionClassAndMethodDefinitionsFromEntry: entry ]! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addClassAndMethodDefinitionsFromDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addClassAndMethodDefinitionsFromEntry: classEntry
	"A class entry here is a ZipDirectoryMember."

	"Would have been convenient to have directory members listing the files they contain."

	| classPropertiesDict classComment entries |
	entries := self zip members
		select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: classEntry fileName ] ].
	(entries detect: [ :entry | self isPropertyFile: entry ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	(entries detect: [ :entry | entry splitFileName last = 'README.md' ] ifNone: [  ])
		ifNotNil: [ :commentEntry | classComment := (self zipEntryContents: commentEntry) contents withSqueakLineEndings ].
	self addClassDefinitionFrom: classPropertiesDict comment: classComment .
	self addMethodDefinitionsForClass: (classPropertiesDict at: 'name') in: entries root: classEntry! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addDefinitionFromEntry: aZipFileMember
	(self definitionFromEntry: aZipFileMember) ifNotNil: [ :def | definitions add: def ]! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addDefinitionFromFile: directoryEntry inDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addExtensionClassAndMethodDefinitionsFromEntry: classEntry
	"A class entry here is a ZipDirectoryMember."

	"Would have been convenient to have directory members listing the files they contain."

	| classPropertiesDict entries |
	entries := self zip members select: [ :entry | entry fileName beginsWith: classEntry fileName ].
	(entries detect: [ :entry | self isPropertyFile: entry ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	self addMethodDefinitionsForClass: (classPropertiesDict at: 'name') in: entries root: classEntry! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addMethodDefinitionsForClass: className in: entries root: aClassEntry
	"Timestamp for methods is recreated from the package info."

	(entries
		select: [ :entry | entry fileName matchesRegex: aClassEntry fileName , '((class)|(instance))' , self slash , '[^' , self slash , ']+.st' ])
		do: [ :methodEntry | 
			| classIsMeta fileStream category source timestamp selector |
			classIsMeta := false.
			(methodEntry fileName beginsWith: aClassEntry fileName , 'class' , self slash)
				ifTrue: [ classIsMeta := true ].
			fileStream := self zipEntryContents: methodEntry.
			category := fileStream nextLine.
			source := fileStream upToEnd.
			selector := self methodSelectorFor: source.
			timestamp := self info author , ' ' , self info date mmddyyyy , ' ' , self info time print24.	" ]"
			definitions
				add:
					(MCMethodDefinition
						className: className
						classIsMeta: classIsMeta
						selector: selector
						category: category
						timeStamp: timestamp
						source: source) ]! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addMethodDefinitionsForClass: className methodProperties: methodProperties in: entries
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
addTraitAndMethodDefinitionsFromEntry: traitEntry
	| classPropertiesDict classComment entries |
	entries := self zip members
		select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: traitEntry fileName ] ].
	(entries detect: [ :entry | self isPropertyFile: entry ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	(entries detect: [ :entry | entry splitFileName last = 'README.md' ] ifNone: [  ])
		ifNotNil: [ :commentEntry | classComment := (self zipEntryContents: commentEntry) contents ]
		ifNil: [ classComment := '' ].
	self addTraitDefinitionFrom: classPropertiesDict comment: classComment withSqueakLineEndings.
	self addMethodDefinitionsForClass: (classPropertiesDict at: 'name') in: entries root: traitEntry! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
commitID
	^ commitID! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
commitID: aString
	commitID := aString! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
definitionFromEntry: aZipFileMember
	| defs reader |
	reader := MCStReader on: (self zipEntryContents: aZipFileMember).
	(defs := reader definitions) size <= 1
		ifFalse: [ self error: 'we should be writing exactly 1 definition per file' ].
	defs size = 0
		ifTrue: [ ^ nil ].
	^ defs first! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
definitionFromFile: directoryEntry inDirectory: aDirectory
	self shouldNotImplement! !

!MCFileTreeGitStReader methodsFor: 'testing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
hasMonticelloMetadata
	"Try to avoid loading the full package :)"
	
	"Since we use a hack to avoid loading the full archive, consider this to be true."
	
	^ true! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
info
	^ info! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
info: aMCVersionInfo
	info := aMCVersionInfo! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
isMethodPropertyFile: entry
	| entryName |
	entryName := entry splitFileName last.
	^ entryName = 'methodProperties.ston' or: [ entryName = 'methodProperties.json']! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
isPropertyFile: entry
	| entryName |
	entryName := entry splitFileName last.
	^ entryName = 'properties.ston' or: [ entryName = 'properties.json']! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
loadDefinitionFor: aMCDefinition
	"Load the definition in this commit for that MCDefinition. Return nil if it is empty (effect of git tracking renames :))"

	| classPropertiesDict entries classEntry |
	definitions := OrderedCollection new.
	self zipForDefinition: aMCDefinition.
	classEntry := packageDirectory , self slash , aMCDefinition className , '.class/'.
	entries := self zip members select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: classEntry ] ].
	entries isEmpty
		ifTrue: [ ^ nil ].
	classPropertiesDict := Dictionary new.
	(entries detect: [ :entry | entry splitFileName last = 'properties.json' ] ifNone: [  ])
		ifNotNil: [ :propertyEntry | classPropertiesDict := MCFileTreeJsonParser parseStream: (self zipEntryContents: propertyEntry) ].
	self
		addMethodDefinitionsForClass: (classPropertiesDict at: 'name')
		in: entries
		root: (self zip members detect: [ :entry | entry isDirectory and: [ entry fileName endsWith: classEntry ] ]).
	^ definitions! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
loadDefinitions
	"Query the zip archive for the relevant data."

	"Add support for scripts in packages."

	| query |
	definitions := OrderedCollection new.
	query := self zip
		membersMatching: packageDirectory , self slash , self monticelloMetaDirName , self slash , 'categories*'.
	query notEmpty
		ifTrue: [ self addDefinitionFromEntry: query first ]
		ifFalse: [ definitions add: (MCOrganizationDefinition categories: {(self packageNameFromPackageDirectory)}) ].
	self addClassAndMethodDefinitions.
	#('initializers*' 'preamble.st' 'postscript.st' 'preambleOfRemoval*' 'postscriptOfRemoval*')
		do: [ :item | 
			query := self zip membersMatching: packageDirectory , self slash , self monticelloMetaDirName , self slash , item.
			query notEmpty
				ifTrue: [ self addDefinitionFromEntry: query first ] ]! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'HilaireFernandes 10/24/2015 16:10'!
loadDependencies
	"Should change that code by the following: force the reading of dependencies the hard way."

	"Here, in fact, what we need to do is to try to change the dependency info because we won't be directly in charge of loading those?"

	| dependencyDir singleGitCommit isSingleCommit |
	dependencyDir := packageDirectory , self slash , self monticelloMetaDirName , self slash , 'dependencies' , self slash.
	singleGitCommit := packageDirectory , self slash , self monticelloMetaDirName , self slash
		, MCFileTreeGitRepository vcsPerPackageProperties.
	isSingleCommit := self zip members anySatisfy: [ :entry | entry fileName = singleGitCommit ].
	dependencies := OrderedCollection new.
	MCCacheRepository uniqueInstance
		cacheAllFileNamesDuring: [ 
			repository
				cacheAllFileNamesDuring: [ 
					(self zip members select: [ :entry | entry isDirectory not and: [ entry fileName beginsWith: dependencyDir ] ])
						do: [ :entry | 
							| name |
							name := entry splitFileName last.
							dependencies
								add:
									(isSingleCommit
										ifTrue: [ MCResolvedVersionDependency package: (MCPackage named: name) fromGit: commitID in: repository ]
										ifFalse: [ 
											MCVersionDependency
												package: (MCPackage named: name)
												info: (self extractInfoFrom: (self parseMember: 'dependencies' , self slash , name)) ]) ] ] ].
	dependencies := dependencies asArray! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
loadVersionInfo: aString
	"If we manage to have a correct string, with some version info, get it there."
	
	| tokens |
	tokens := [ self scanner scan: (ReadStream on: aString) ]
		on: Error
		do: [ #() ].
	^ info := self extractInfoFrom: (self associate: tokens)! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
packageDirectory: aDirectoryName
	"Set the package name at the same time, no need to read the metadata anymore."

	packageDirectory := aDirectoryName.
	package := MCPackage named: self packageNameFromPackageDirectory! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
packageNameFromPackageDirectory
	| filename |
	filename := self fileUtils directoryName: packageDirectory asFileReference.
	^ filename copyFrom: 1 to: (filename lastIndexOf: $.) - 1! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
parseMember: fileName
	"Parse a metadata member?"

	"Improvement: try to use the default zip file, since we are retrieving a lot of the metadata info by other means (version, author, timestamp, package name)."

	| monticelloMetaEntry tokens |
	monticelloMetaEntry := packageDirectory , self slash , self monticelloMetaDirName , self slash , fileName.
	tokens := [ self scanner scan: (self zipEntryContents: (self zip memberNamed: monticelloMetaEntry)) ]
		on: Error
		do: [ #() ].
	^ self associate: tokens! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
repository
	^ repository! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
repository: anObject
	repository := anObject! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
slash
	^ '/'! !

!MCFileTreeGitStReader methodsFor: 'private' stamp: 'HilaireFernandes 10/24/2015 16:16'!
vcsCommand: aCommandString in: aDirectory
	^ MCFileTreeGitRepository runVcsCommand: aCommandString in: aDirectory! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:16'!
zip
	"Get the zip based on the commitID."

	zip isNil
		ifTrue: [ 
			zip := ZipArchive new.
			zip
				readFrom:
					(self
						vcsCommand:
							{'archive'.
							'--format=zip'.
							(commitID hex).
							packageDirectory}
						in: stream) asByteArray readStream ].
	^ zip! !

!MCFileTreeGitStReader methodsFor: 'utilities' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
zipEntryContents: aZipFileMember
	"Try to get the correct encoding for UTF8."

	^ ReadStream on: aZipFileMember contents convertFromSystemString! !

!MCFileTreeGitStReader methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 16:16'!
zipForDefinition: aMCDefinition
	"We suppose we are trying to load a specific method definition. Recover the relevant archive as needed."

	"Ask in the archive, the method file, and the class properties file."

	| classDirectory methodFile classPropertiesFile |
	self assert: aMCDefinition isMethodDefinition.
	zip := ZipArchive new.
	classDirectory := packageDirectory , self slash , aMCDefinition className
		,
			(aMCDefinition isExtensionMethod
				ifTrue: [ '.extension' ]
				ifFalse: [ '.class' ]).
	methodFile := classDirectory , self slash
		,
			(aMCDefinition classIsMeta
				ifTrue: [ 'class' ]
				ifFalse: [ 'instance' ]) , self slash , (MCFileTreeStCypressWriter new fileNameForSelector: aMCDefinition selector)
		, '.st'.
	classPropertiesFile := classDirectory , self slash , 'properties.json'.
	zip
		readFrom:
			(self
				vcsCommand:
					{'archive'.
					'--format=zip'.
					(commitID hex).
					methodFile.
					classPropertiesFile}
				in: stream) asByteArray readStream.
	^ zip! !

!MCVersionInfo methodsFor: '*monticellofiletree-git' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
id: anUUID
	id := anUUID! !

!SettingDeclaration methodsFor: '*monticellofiletree-git' stamp: 'ThierryGoubier 10/20/2015 14:49:01'!
inputWidgetForFile

	^ self inputWidgetForFileOrDirectoryWithAction: #chooseFilePath.! !

Object subclass: #VcsFileTreePackageEntry
	instanceVariableNames: 'packageDirectory info commitID message author timeStamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!VcsFileTreePackageEntry commentStamp: 'HilaireFernandes 10/24/2015 15:45' prior: 0!
The abstract VcsFileTreePackage represent a package in a filetree://repository

Instance Variables
	info:		<MCVersion>
	packageDirectory:		<FileReference>!

VcsFileTreePackageEntry subclass: #GitFileTreePackageEntry
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Git'!

!GitFileTreePackageEntry commentStamp: '<historical>' prior: 0!
A GitFileTreePackage represent a package in a gitfiletree://repository

Instance Variables
	info:		<MCVersion>
	packageDirectory:		<FileReference>!

!GitFileTreePackageEntry class methodsFor: 'utility' stamp: 'HilaireFernandes 10/24/2015 15:50'!
uuidFromVcsSHA: cid package: aPackageName
	"Build a unique SHA1 hash out of the git commit ID and package name."

	"Should ensure:
		repeatability... same UUID everytime for a git commitID and a package name.
		unicity... no two different packages, even with the same commitID, have the same UUID."

	^ self uuidFromSHA: (SHA1 new hashStream: (ReadStream on: cid hex , aPackageName))! !

!VcsFileTreePackageEntry class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 15:47'!
on: aPackageDirectory id: aCommitID author: aString message: aMessage timeStamp: aDateAndTime
	^ self new
		packageDirectory: aPackageDirectory;
		commitID: aCommitID;
		author: aString;
		message: aMessage;
		timeStamp: aDateAndTime! !

!VcsFileTreePackageEntry class methodsFor: 'instance creation' stamp: 'HilaireFernandes 10/24/2015 15:47'!
on: aPackageDirectory id: aCommitID info: info
	^ self new
		packageDirectory: aPackageDirectory;
		info: info;
		commitID: aCommitID! !

!VcsFileTreePackageEntry class methodsFor: 'utility' stamp: 'HilaireFernandes 10/24/2015 15:47'!
uuidFromSHA: cid
	"Return an adequate uuid from a git commit ID."

	^ UUID nilUUID
		in: [ :uuid |
			1 to: 16 do: [ :i | uuid at: i put: (cid at: i) ].
			uuid at: 9 put: (16r80 bitOr: ((uuid at: 9) bitAnd: 16r3F)).	"Type"
			uuid at: 7 put: (16r50 bitOr: ((uuid at: 7) bitAnd: 16r0F)).	"Version 5 : SHA-1 Namespace (git)."
			uuid ]! !

!VcsFileTreePackageEntry class methodsFor: 'utility' stamp: 'HilaireFernandes 10/24/2015 15:48'!
uuidFromVcsSHA: cid package: aPackageName
	self subclassResponsibility! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:45'!
author: aString
	author := aString! !

!VcsFileTreePackageEntry methodsFor: 'building' stamp: 'HilaireFernandes 10/24/2015 15:49'!
buildInfoWith: aList startingAt: anIndex version: aNumber ancestry: aDictionary
	"Build the MCVersionInfo we need with all the relevant data."

	| ancestors aIDs stack packageName |
	packageName := packageDirectory copyUpToLast: $..
	ancestors := OrderedCollection new.
	stack := Set new.
	aIDs := Set new.
	stack addAll: (aDictionary at: commitID).
	[ stack isEmpty ]
		whileFalse: [ 
			| newStack j found |
			newStack := Set new.
			stack
				do: [ :current | 
					j := anIndex.
					found := false.
					(aIDs includes: current)
						ifTrue: [ found := true ].
					[ j <= aList size and: [ found not ] ]
						whileTrue: [ 
							(aList at: j) commitID = current
								ifTrue: [ 
									ancestors add: (aList at: j) info.
									aIDs add: current.
									found := true ].
							j := j + 1 ].
					found
						ifFalse: [ 
							aIDs add: current.
							newStack addAll: (aDictionary at: current) ] ].
			stack := newStack ].
	info := MCVersionInfo
		name: packageName , '-' , author , '.' , aNumber printString
		id: (self class uuidFromVcsSHA: commitID package: packageName)
		message: message
		date: timeStamp asDate
		time: timeStamp asTime
		author: author
		ancestors: ancestors asArray! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
commitID
	^ commitID! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
commitID: aString
	commitID := aString! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
info
	^ info! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
info: aMCVersion
	info := aMCVersion! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
message: aString
	message := aString! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
name
	^ info name! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
packageDirectory
	^ packageDirectory! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
packageDirectory: aPackageDirectory
	packageDirectory := aPackageDirectory! !

!VcsFileTreePackageEntry methodsFor: 'printing' stamp: 'HilaireFernandes 10/24/2015 15:46'!
printString
	"Get the name of the package."

	^ info name! !

!VcsFileTreePackageEntry methodsFor: 'version protocol' stamp: 'HilaireFernandes 10/24/2015 15:46'!
summary
	^ String
		streamContents: [ :s | 
			s nextPutAll: info summaryHeader.
			s
				cr;
				cr;
				nextPutAll: info message ]! !

!VcsFileTreePackageEntry methodsFor: 'accessing' stamp: 'HilaireFernandes 10/24/2015 15:47'!
timeStamp: aDateAndTime
	timeStamp := aDateAndTime! !

!VcsFileTreePackageEntry methodsFor: 'version protocol' stamp: 'HilaireFernandes 10/24/2015 15:47'!
version
	^ self! !
